"use strict";

exports.any = void 0;
exports.anyOf = anyOf;
exports.charClass = charClass;
exports.charRange = charRange;
exports.inverted = exports.digit = void 0;
exports.negated = negated;
exports.word = exports.whitespace = exports.notWord = exports.notWhitespace = exports.notDigit = exports.nonWord = exports.nonWhitespace = exports.nonDigit = void 0;
const any = exports.any = {
  precedence: 'atom',
  pattern: '.'
};
const digit = exports.digit = {
  type: 'characterClass',
  escape: '\\d',
  encode: encodeCharacterClass
};
const nonDigit = exports.nonDigit = {
  type: 'characterClass',
  escape: '\\D',
  encode: encodeCharacterClass
};
const word = exports.word = {
  type: 'characterClass',
  escape: '\\w',
  encode: encodeCharacterClass
};
const nonWord = exports.nonWord = {
  type: 'characterClass',
  escape: '\\W',
  encode: encodeCharacterClass
};
const whitespace = exports.whitespace = {
  type: 'characterClass',
  escape: '\\s',
  encode: encodeCharacterClass
};
const nonWhitespace = exports.nonWhitespace = {
  type: 'characterClass',
  escape: '\\S',
  encode: encodeCharacterClass
};
const notDigit = exports.notDigit = nonDigit;
const notWord = exports.notWord = nonWord;
const notWhitespace = exports.notWhitespace = nonWhitespace;
function charClass(...elements) {
  if (elements.some(e => e.isNegated)) {
    throw new Error('`charClass` should receive only non-negated character classes');
  }
  if (elements.length === 1) {
    return elements[0];
  }
  return {
    type: 'characterClass',
    chars: elements.map(c => getAllChars(c) ?? []).flat(),
    ranges: elements.map(c => c.ranges ?? []).flat(),
    encode: encodeCharacterClass
  };
}
function charRange(start, end) {
  if (start.length !== 1) {
    throw new Error('`charRange` should receive only single character `start` string');
  }
  if (end.length !== 1) {
    throw new Error('`charRange` should receive only single character `end` string');
  }
  if (start > end) {
    throw new Error('`start` should be before or equal to `end`');
  }
  return {
    type: 'characterClass',
    ranges: [{
      start,
      end
    }],
    encode: encodeCharacterClass
  };
}
function anyOf(characters) {
  const chars = characters.split('').map(c => escapeForCharacterClass(c));
  if (chars.length === 0) {
    throw new Error('`anyOf` should received at least one character');
  }
  return {
    type: 'characterClass',
    chars,
    encode: encodeCharacterClass
  };
}
function negated(element) {
  return {
    type: 'characterClass',
    escape: element.escape,
    chars: element.chars,
    ranges: element.ranges,
    isNegated: !element.isNegated,
    encode: encodeCharacterClass
  };
}
const inverted = exports.inverted = negated;
function encodeCharacterClass() {
  if (this.escape === undefined && !this.chars?.length && !this.ranges?.length) {
    throw new Error('Character class should contain at least one character or character range');
  }
  if (this.escape !== undefined && !this.chars?.length && !this.ranges?.length && !this.isNegated) {
    return {
      precedence: 'atom',
      pattern: this.escape
    };
  }
  const allChars = getAllChars(this) ?? [];
  const hyphen = allChars.includes('-') ? '-' : '';
  const caret = allChars.includes('^') ? '^' : '';
  const otherChars = allChars.filter(c => c !== '-' && c !== '^').join('');
  const ranges = this.ranges?.map(({
    start,
    end
  }) => `${start}-${end}`).join('') ?? '';
  const negation = this.isNegated ? '^' : '';
  let pattern = `[${negation}${ranges}${otherChars}${caret}${hyphen}]`;
  if (pattern === '[^-]') pattern = '[\\^-]';
  return {
    precedence: 'atom',
    pattern
  };
}
function escapeForCharacterClass(text) {
  return text.replace(/[\]\\]/g, '\\$&');
}
function getAllChars(characterClass) {
  if (characterClass.escape === undefined) {
    return characterClass.chars;
  }
  return [characterClass.escape, ...(characterClass.chars ?? [])];
}
//# sourceMappingURL=character-class.js.map