export const any = {
  precedence: 'atom',
  pattern: '.'
};
export const digit = {
  type: 'characterClass',
  escape: '\\d',
  encode: encodeCharacterClass
};
export const nonDigit = {
  type: 'characterClass',
  escape: '\\D',
  encode: encodeCharacterClass
};
export const word = {
  type: 'characterClass',
  escape: '\\w',
  encode: encodeCharacterClass
};
export const nonWord = {
  type: 'characterClass',
  escape: '\\W',
  encode: encodeCharacterClass
};
export const whitespace = {
  type: 'characterClass',
  escape: '\\s',
  encode: encodeCharacterClass
};
export const nonWhitespace = {
  type: 'characterClass',
  escape: '\\S',
  encode: encodeCharacterClass
};
export const notDigit = nonDigit;
export const notWord = nonWord;
export const notWhitespace = nonWhitespace;
export function charClass(...elements) {
  if (elements.some(e => e.isNegated)) {
    throw new Error('`charClass` should receive only non-negated character classes');
  }
  if (elements.length === 1) {
    return elements[0];
  }
  return {
    type: 'characterClass',
    chars: elements.map(c => getAllChars(c) ?? []).flat(),
    ranges: elements.map(c => c.ranges ?? []).flat(),
    encode: encodeCharacterClass
  };
}
export function charRange(start, end) {
  if (start.length !== 1) {
    throw new Error('`charRange` should receive only single character `start` string');
  }
  if (end.length !== 1) {
    throw new Error('`charRange` should receive only single character `end` string');
  }
  if (start > end) {
    throw new Error('`start` should be before or equal to `end`');
  }
  return {
    type: 'characterClass',
    ranges: [{
      start,
      end
    }],
    encode: encodeCharacterClass
  };
}
export function anyOf(characters) {
  const chars = characters.split('').map(c => escapeForCharacterClass(c));
  if (chars.length === 0) {
    throw new Error('`anyOf` should received at least one character');
  }
  return {
    type: 'characterClass',
    chars,
    encode: encodeCharacterClass
  };
}
export function negated(element) {
  return {
    type: 'characterClass',
    escape: element.escape,
    chars: element.chars,
    ranges: element.ranges,
    isNegated: !element.isNegated,
    encode: encodeCharacterClass
  };
}
export const inverted = negated;
function encodeCharacterClass() {
  if (this.escape === undefined && !this.chars?.length && !this.ranges?.length) {
    throw new Error('Character class should contain at least one character or character range');
  }
  if (this.escape !== undefined && !this.chars?.length && !this.ranges?.length && !this.isNegated) {
    return {
      precedence: 'atom',
      pattern: this.escape
    };
  }
  const allChars = getAllChars(this) ?? [];
  const hyphen = allChars.includes('-') ? '-' : '';
  const caret = allChars.includes('^') ? '^' : '';
  const otherChars = allChars.filter(c => c !== '-' && c !== '^').join('');
  const ranges = this.ranges?.map(({
    start,
    end
  }) => `${start}-${end}`).join('') ?? '';
  const negation = this.isNegated ? '^' : '';
  let pattern = `[${negation}${ranges}${otherChars}${caret}${hyphen}]`;
  if (pattern === '[^-]') pattern = '[\\^-]';
  return {
    precedence: 'atom',
    pattern
  };
}
function escapeForCharacterClass(text) {
  return text.replace(/[\]\\]/g, '\\$&');
}
function getAllChars(characterClass) {
  if (characterClass.escape === undefined) {
    return characterClass.chars;
  }
  return [characterClass.escape, ...(characterClass.chars ?? [])];
}
//# sourceMappingURL=character-class.mjs.map